const rootEventListeners = new Map();
let isRootListenerAdded = false;
function addRootEventListener(key, listener) {
    console.log(`adding root listener for ${key}`);
    rootEventListeners.set(key, listener);
}
function removeRootEventListener(key) {
    rootEventListeners.delete(key);
}
function onStorageChange(event) {
    if (!event.key)
        return;
    const listener = rootEventListeners.get(event.key);
    if (!listener)
        throw new Error(`No listener found for key ${event.key}`);
    listener(event.newValue);
}
function ensureRootListener() {
    if (!isRootListenerAdded && typeof window !== 'undefined') {
        window.addEventListener('storage', onStorageChange);
        isRootListenerAdded = true;
    }
}
export function persist(key, initialValue, options = {}) {
    const { storage = localStorage, serializer = JSON.stringify, deserializer = JSON.parse, afterInit = (value) => value, syncTabs = true, root = false, } = options;
    const afterGet = (value) => {
        if (!value)
            return initialValue;
        return deserializer(value);
    };
    const beforeSet = (value) => {
        return serializer(value);
    };
    let state = $state(afterInit(afterGet(storage.getItem(key))));
    const syncWithStorage = () => {
        $effect(() => {
            storage.setItem(key, beforeSet(state));
        });
    };
    let cleanup = () => { };
    if (root) {
        cleanup = $effect.root(() => {
            syncWithStorage();
            if (syncTabs) {
                addRootEventListener(key, (value) => {
                    state = afterGet(value);
                });
                ensureRootListener();
                return () => removeRootEventListener(key);
            }
        });
    }
    else {
        syncWithStorage();
        if (syncTabs) {
            const listener = (event) => {
                if (event.key === key && event.newValue !== null) {
                    state = afterGet(event.newValue);
                }
            };
            $effect(() => {
                window.addEventListener('storage', listener);
                return () => window.removeEventListener('storage', listener);
            });
        }
    }
    return {
        get value() {
            return state;
        },
        set value(value) {
            state = value;
        },
        cleanup,
        reset: () => {
            state = initialValue;
        },
    };
}
